if(ThingsToDo2 == nil) then
	ThingsToDo2 = {}
end
local TTD2 = ThingsToDo2

--[[
This lib implements a data table containing faction-information. the client is expected to add factions
first by calling addByID() and then access the information using getByID().
the internal data can be updated on demand by calling updateByID().
Note that simply calling getByID() will NOT update the internal data.

The reason for this buffering is that it allowes me to have different data fields than the API
without creating new tables on every get() call.

The drawback is that memory consumption will rise the more faction data is requested and that memory will
never be freed until a /reload.
Since we are talking about simple integer and short string data here, this seems negligible compared
to the runtime performance improvement or the alternative of coping with the API fields directly.
]]

local super = TTD2.Templates.Collection
TTD2.Tools.Faction = super:new()
local this = TTD2.Tools.Faction

function this:UpdateByID(ID)
	
	local Facton = self:GetByID(ID)
	
	local FactionInfo = { GetFactionInfoByID(ID) }
	
	Facton.Name			= FactionInfo[1]
    Facton.Description	= FactionInfo[2]
	
	if(C_Reputation.IsFactionParagon(ID) == true) then
		local ParagonInfo = { C_Reputation.GetFactionParagonInfo(ID) }
		Facton.Paragon = Facton.Paragon or {}
		
		Facton.Paragon.CurrentRep			= ParagonInfo[1]
		Facton.Paragon.NextRep				= ParagonInfo[2]
		Facton.Paragon.RewardQuestID		= ParagonInfo[3]
		Facton.Paragon.HasRewardPending		= ParagonInfo[4]
		
		Facton.Paragon.NumRewardsObtained	= 0
		while(Facton.Paragon.CurrentRep > Facton.Paragon.NextRep) do
			Facton.Paragon.NumRewardsObtained = Facton.Paragon.NumRewardsObtained + 1
			Facton.Paragon.CurrentRep = Facton.Paragon.CurrentRep - Facton.Paragon.NextRep
		end
	else
		Facton.Paragon = nil
	end
	
	return
end
