ThingsToDo2 = ThingsToDo2 or {}
local TTD2 = ThingsToDo2

TTD2.Templates = TTD2.Templates or {}
local Templates = TTD2.Templates

TTD2.Tools = TTD2.Tools or {}
local Tools = TTD2.Tools

--[[
The purpose of this tool is to handle our SavedVariables. Most importantly to default values not present in SavedVars
and remove deprecated values from SavedVars.
]]

local super = Templates.Base
Tools.SavedData = super:new()
local this = Tools.SavedData

this.private = {}
function this.private:LoadSetting(Setting, Default)
	if(	(ThingsToDo2_CharacterData ~= nil) and 
		(ThingsToDo2_CharacterData.Settings ~= nil) and 
		(ThingsToDo2_CharacterData.Settings[Setting] ~= nil) ) then
		self.Settings[Setting] = ThingsToDo2_CharacterData.Settings[Setting]
	else
		self.Settings[Setting] = Default
	end
end

function this.private:LoadValue(Value, Default)
	if(	(ThingsToDo2_CharacterData ~= nil) and 
		(ThingsToDo2_CharacterData[Value] ~= nil) ) then
		self.Values[Value] = ThingsToDo2_CharacterData[Value]
	else
		self.Values[Value] = Default
	end
end

function this:LoadFromGlobals()
	self.private.Values			= self.private.Values		or {} --ThingsToDo2CharacterData
	self.private.Settings		= self.private.Settings		or {} --ThingsToDo2CharacterData.Settings
	self.private.AccountTable	= ThingsToDo2_AccountData	or {} --ThingsToDo2_AccountData
	
	self.private:LoadSetting("PrintCompleted", false)
	self.private:LoadSetting("PrintDone", false)
	self.private:LoadSetting("RareSources", true)
	self.private:LoadSetting("VerboseTomTom", false)
	self.private:LoadSetting("HideMinimapButton", false)
	self.private:LoadSetting("CharacterAchievements", false)
	self.private:LoadSetting("ParagonEmissary", true)
	self.private:LoadSetting("AchievementCompletionist", false)
	self.private:LoadSetting("XMogCompletionist", false)
	self.private:LoadSetting("PetCompletionist", false)
	self.private:LoadSetting("GoldenCages", false)
	self.private:LoadSetting("ArchaeologyAutoWaypoint", false)
	self.private:LoadSetting("AllowSecureButtons", true)
	self.private:LoadSetting("MPlusFeatHunter", true)
	self.private:LoadSetting("ResourcefulBonusRoll", false)
	self.private:LoadSetting("MPlusAccount", false)
	self.private:LoadSetting("MPlusAccountAlwaysShowStones", false)
	
	self.private:LoadValue("MinimapPosition", 0.0)
	
	--we read all the data we need, now point the SavedVariables onto our new tables to make sure if we ever deprecate options they will be properly removed from users of older versions
	ThingsToDo2_CharacterData = self.private.Values
	ThingsToDo2_CharacterData.Settings = self.private.Settings
	
	return
end

function this:ClearAccountTable()
	Tools.Chat:Warning("clearing all account SavedVariables")
	
	self.private.AccountTable = {}
	ThingsToDo2_AccountData = self.private.AccountTable
	
	return
end

function this:GetSetting(Setting)
	if(self.private.Settings[Setting] == nil) then
		Tools.Chat:Error("trying to read invalid setting: "..Setting)
	end
	
	return self.private.Settings[Setting]
end

function this:SetSetting(Setting, Data)
	if(self.private.Settings[Setting] == nil) then
		Tools.Chat:Error("trying to write invalid setting: "..Setting)
	end
	
	self.private.Settings[Setting] = Data
end

function this:GetValue(Value)
	if(self.private.Values[Value] == nil) then
		Tools.Chat:Error("trying to read invalid value: "..Value)
	end
	
	return self.private.Values[Value]
end

function this:SetValue(Value, Data)
	if(self.private.Values[Value] == nil) then
		Tools.Chat:Error("trying to write invalid value: "..Value)
	end
	
	self.private.Values[Value] = Data
end

function this:GetAccountTable()
	return self.private.AccountTable
end
